/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.client;

import java.util.Optional;
import java.util.logging.Level;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.operator.Config;
import de.ozgcloud.operator.keycloak.OzgCustomResourceStatus;
import io.javaoperatorsdk.operator.api.reconciler.Context;
import io.javaoperatorsdk.operator.api.reconciler.ControllerConfiguration;
import io.javaoperatorsdk.operator.api.reconciler.Reconciler;
import io.javaoperatorsdk.operator.api.reconciler.UpdateControl;
import lombok.extern.java.Log;

@ControllerConfiguration
@Component
@Log
public class KeycloakClientReconciler implements Reconciler<OzgKeycloakClient> {

	@Autowired
	private KeycloakClientService service;

	@Autowired
	private KeycloakClientPreconditionService preconditionService;

	@Override
	public UpdateControl<OzgKeycloakClient> reconcile(OzgKeycloakClient resource, Context<OzgKeycloakClient> context) {

		try {
			String crdName = resource.getMetadata().getName();

			log.info("Reconcile KeycloakClient " + crdName);

			Optional<String> preconditionError = preconditionService.getReconcilePreconditionErrors(resource);
			if (preconditionError.isPresent()) {
				return buildStatusInProgress(resource, preconditionError.get());
			}

			service.createOrUpdateClient(resource.getSpec(), resource.getMetadata().getNamespace());

			return buildStatusOk(resource);
		} catch (Exception e) {
			log.log(Level.SEVERE,
					"Could not reconcile client " + resource.getMetadata().getName() + " in namespace " + resource.getMetadata().getNamespace() + ": "
							+ e.getMessage(),
					e);
			resource.setStatus(OzgKeycloakClientStatus.builder().status(OzgCustomResourceStatus.ERROR).message(e.getMessage()).build());
			return UpdateControl.updateStatus(resource).rescheduleAfter(Config.RECONCILER_RETRY_SECONDS);
		}
	}

	private UpdateControl<OzgKeycloakClient> buildStatusOk(OzgKeycloakClient resource) {
		resource.setStatus(OzgKeycloakClientStatus.builder().status(OzgCustomResourceStatus.OK).message(null).build());
		return UpdateControl.updateStatus(resource);
	}

	private UpdateControl<OzgKeycloakClient> buildStatusInProgress(OzgKeycloakClient resource, String errorMessage) {
		log.log(Level.INFO,
				"Could not yet reconcile client " + resource.getMetadata().getName() + " in namespace " + resource.getMetadata().getNamespace() + ":"
						+ errorMessage);
		resource.setStatus(OzgKeycloakClientStatus.builder().status(OzgCustomResourceStatus.IN_PROGRESS).message(errorMessage).build());
		return UpdateControl.updateStatus(resource).rescheduleAfter(Config.RECONCILER_RETRY_SECONDS);
	}
}
