package de.ozgcloud.operator.keycloak.client;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;

import org.apache.commons.lang3.StringUtils;
import org.keycloak.representations.idm.ProtocolMapperRepresentation;
import org.springframework.stereotype.Component;

import lombok.extern.java.Log;

@Log
@Component
class ProtocolMapperRepresentationHelper {

	static final String OPENID_CONNECT_PROTOCOL = "openid-connect";

	static final String USER_ATTRIBUTE_PROTOCOL_MAPPER = "oidc-usermodel-attribute-mapper";
	static final String USER_CLIENT_ROLE_PROTOCOL_MAPPER = "oidc-usermodel-client-role-mapper";

	static final String CONFIG_ACCESS_TOKEN_CLAIM_KEY = "access.token.claim";
	static final String CONFIG_AGGREGATE_ATTRS_KEY = "aggregate.attrs";
	static final String CONFIG_CLAIM_NAME_KEY = "claim.name";
	static final String CONFIG_ID_TOKEN_CLAIM_KEY = "id.token.claim";
	static final String CONFIG_JSON_TYPE_LABEL_KEY = "jsonType.label";
	static final String CONFIG_MULTIVALUED_KEY = "multivalued";
	static final String CONFIG_USER_ATTRIBUTE_KEY = "user.attribute";
	static final String CONFIG_USERINFO_TOKEN_CLAIM_KEY = "userinfo.token.claim";

	static final String CONFIG_USER_MODEL_CLIENT_ROLE_MAPPING_CLIENT_ID = "usermodel.clientRoleMapping.clientId";
	static final String CONFIG_USER_MODEL_CLIENT_ROLE_MAPPING_ROLE_PREFIX = "usermodel.clientRoleMapping.rolePrefix";

	static final String ORGANISATIONS_EINHEIT_ID_MAPPER_NAME = "organisationseinheitIdMapper";
	static final String OZGCLOUD_USER_ID_MAPPER_NAME = "ozgCloudUserId";
	static final String ORGANISATIONS_EINHEIT_ID_LDAP_MAPPER_NAME = "organisationseinheitIdLdapMapper";
	static final String CLIENT_ROLES_MAPPER_NAME = "client roles";

	public ProtocolMapperRepresentation createOrganisationsEinheitIdMapper() {
		log.log(Level.FINE, "Create createOrganisationsEinheitIdMapper...");
		return createUserAttributeMapper(ORGANISATIONS_EINHEIT_ID_MAPPER_NAME, buildOrganisationsEinheitMapperConfig());
	}

	Map<String, String> buildOrganisationsEinheitMapperConfig() {
		var config = new HashMap<String, String>();
		config.put(CONFIG_ACCESS_TOKEN_CLAIM_KEY, "true");
		config.put(CONFIG_AGGREGATE_ATTRS_KEY, "true");
		config.put(CONFIG_CLAIM_NAME_KEY, "organisationseinheitId");
		config.put(CONFIG_ID_TOKEN_CLAIM_KEY, "true");
		config.put(CONFIG_JSON_TYPE_LABEL_KEY, "int");
		config.put(CONFIG_MULTIVALUED_KEY, "true");
		config.put(CONFIG_USER_ATTRIBUTE_KEY, "organisationseinheitId");
		config.put(CONFIG_USERINFO_TOKEN_CLAIM_KEY, "true");
		return config;
	}

	public ProtocolMapperRepresentation createOzgCloudUserIdMapper() {
		log.log(Level.FINE, "Create createOzgCloudUserIdMapper...");
		return createUserAttributeMapper(OZGCLOUD_USER_ID_MAPPER_NAME, buildOzgCloudUserIdMapperConfig());
	}

	Map<String, String> buildOzgCloudUserIdMapperConfig() {
		var config = new HashMap<String, String>();
		config.put(CONFIG_ACCESS_TOKEN_CLAIM_KEY, "true");
		config.put(CONFIG_AGGREGATE_ATTRS_KEY, "false");
		config.put(CONFIG_CLAIM_NAME_KEY, "ozgCloudUserId");
		config.put(CONFIG_ID_TOKEN_CLAIM_KEY, "true");
		config.put(CONFIG_JSON_TYPE_LABEL_KEY, StringUtils.EMPTY);
		config.put(CONFIG_MULTIVALUED_KEY, "false");
		config.put(CONFIG_USER_ATTRIBUTE_KEY, "ozgCloudUserId");
		config.put(CONFIG_USERINFO_TOKEN_CLAIM_KEY, "true");
		return config;
	}

	public ProtocolMapperRepresentation createOrganisationeEinheitIdLdapMapper() {
		log.log(Level.FINE, "Create createOrganisationeEinheitIdLdapMapper...");
		return createUserAttributeMapper(ORGANISATIONS_EINHEIT_ID_LDAP_MAPPER_NAME, buildOrganisationsEinheitIdLdapMapperConfig());
	}

	private ProtocolMapperRepresentation createUserAttributeMapper(String name, Map<String, String> config) {
		var mapper = new ProtocolMapperRepresentation();
		mapper.setName(name);
		mapper.setProtocol(OPENID_CONNECT_PROTOCOL);
		mapper.setProtocolMapper(USER_ATTRIBUTE_PROTOCOL_MAPPER);
		mapper.setConfig(config);

		return mapper;
	}

	Map<String, String> buildOrganisationsEinheitIdLdapMapperConfig() {
		var config = new HashMap<String, String>();
		config.put(CONFIG_ACCESS_TOKEN_CLAIM_KEY, "true");
		config.put(CONFIG_AGGREGATE_ATTRS_KEY, "true");
		config.put(CONFIG_CLAIM_NAME_KEY, "organisationseinheitId");
		config.put(CONFIG_ID_TOKEN_CLAIM_KEY, "true");
		config.put(CONFIG_JSON_TYPE_LABEL_KEY, "int");
		config.put(CONFIG_MULTIVALUED_KEY, "true");
		config.put(CONFIG_USER_ATTRIBUTE_KEY, "extensionAttribute1");
		config.put(CONFIG_USERINFO_TOKEN_CLAIM_KEY, "true");
		return config;
	}

	public ProtocolMapperRepresentation createClientRolesMapper() {
		log.log(Level.FINE, "Create createClientRolesMapper...");
		return createUserClientRoleMapper(CLIENT_ROLES_MAPPER_NAME, buildClientRolesMapperConfig());
	}

	private ProtocolMapperRepresentation createUserClientRoleMapper(String name, Map<String, String> config) {
		var mapper = new ProtocolMapperRepresentation();
		mapper.setName(name);
		mapper.setProtocol(OPENID_CONNECT_PROTOCOL);
		mapper.setProtocolMapper(USER_CLIENT_ROLE_PROTOCOL_MAPPER);
		mapper.setConfig(config);

		return mapper;
	}

	Map<String, String> buildClientRolesMapperConfig() {
		var config = new HashMap<String, String>();
		config.put(CONFIG_ACCESS_TOKEN_CLAIM_KEY, "true");
		config.put(CONFIG_CLAIM_NAME_KEY, "resource_access.${client_id}.roles");
		config.put(CONFIG_ID_TOKEN_CLAIM_KEY, "false");
		config.put(CONFIG_JSON_TYPE_LABEL_KEY, StringUtils.EMPTY);
		config.put(CONFIG_MULTIVALUED_KEY, "true");
		config.put(CONFIG_USER_MODEL_CLIENT_ROLE_MAPPING_CLIENT_ID, StringUtils.EMPTY);
		config.put(CONFIG_USER_MODEL_CLIENT_ROLE_MAPPING_ROLE_PREFIX, StringUtils.EMPTY);
		return config;
	}
}