/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.keycloak.admin.client.Keycloak;
import org.keycloak.admin.client.resource.ClientResource;
import org.keycloak.admin.client.resource.ClientsResource;
import org.keycloak.admin.client.resource.GroupsResource;
import org.keycloak.admin.client.resource.RealmResource;
import org.keycloak.admin.client.resource.RealmsResource;
import org.keycloak.admin.client.resource.RolesResource;
import org.keycloak.representations.idm.ClientRepresentation;
import org.keycloak.representations.idm.GroupRepresentation;
import org.keycloak.representations.idm.RoleRepresentation;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.operator.keycloak.realm.RealmRepresentationTestFactory;
import de.ozgcloud.operator.keycloak.user.UserRepresentationTestFactory;

class KeycloakGenericRemoteServiceTest {

	private static final String REALM = "TestNamespace";
	private static final String CLIENT_ID = "TestClientID";
	private static final String REAL_CLIENT_ID = "RealTestClientID";

	@Spy
	@InjectMocks
	private KeycloakGenericRemoteService service;

	@Mock
	private Keycloak keycloak;
	@Mock
	private RealmResource realmResource;
	@Mock
	private ClientsResource clientsResource;
	@Mock
	private ClientRepresentation clientRepresentation;
	@Mock
	private RealmsResource realmsResource;
	@Mock
	private GroupsResource groupsResource;
	@Mock
	private GroupRepresentation groupRepresentation;
	@Mock
	private ClientResource clientResource;
	@Mock
	private RolesResource rolesResource;
	@Mock
	private RoleRepresentation roleRepresentation;

	@Nested
	class TestGetKeycloakClientById {

		@BeforeEach
		void init() {
			when(keycloak.realm(REALM)).thenReturn(realmResource);
			when(realmResource.clients()).thenReturn(clientsResource);
			when(clientsResource.findAll()).thenReturn(List.of(clientRepresentation));
			when(clientRepresentation.getClientId()).thenReturn(CLIENT_ID);
		}

		@Test
		void shouldGetClientFromKeycloak() {

			var client = service.getByClientId(CLIENT_ID, REALM);

			assertThat(client).isNotEmpty().contains(clientRepresentation);
		}
	}

	@Nested
	class TestRealmExists {

		@BeforeEach
		void init() {
			when(keycloak.realms()).thenReturn(realmsResource);
		}

		@Test
		void shouldReturnFalseOnRealmNotExists() {
			when(realmsResource.findAll()).thenReturn(Collections.emptyList());

			var response = service.realmExists(RealmRepresentationTestFactory.NAME);

			assertThat(response).isFalse();
		}

		@Test
		void shouldReturnTrueOnRealmNotExists() {
			when(realmsResource.findAll()).thenReturn(List.of(RealmRepresentationTestFactory.create()));

			var response = service.realmExists(RealmRepresentationTestFactory.NAME);

			assertThat(response).isTrue();
		}
	}

	@Nested
	class TestGroupExists {

		private static final String GROUP_NAME = "GroupName";

		@BeforeEach
		void init() {
			when(keycloak.realm(REALM)).thenReturn(realmResource);
			when(realmResource.groups()).thenReturn(groupsResource);
		}

		@Test
		void shouldReturnFalseIfGroupNotExists() {
			when(groupsResource.groups()).thenReturn(Collections.emptyList());

			boolean rsp = service.groupExists(GROUP_NAME, REALM);

			assertThat(rsp).isFalse();
		}

		@Test
		void shouldReturnTrueIfGroupExists() {
			when(groupsResource.groups()).thenReturn(List.of(groupRepresentation));
			when(groupRepresentation.getName()).thenReturn(GROUP_NAME);

			boolean rsp = service.groupExists(GROUP_NAME, REALM);

			assertThat(rsp).isTrue();
		}
	}

	@Nested
	class TestToClientRole {

		@BeforeEach
		void init() {
			when(keycloak.realm(REALM)).thenReturn(realmResource);
			when(realmResource.clients()).thenReturn(clientsResource);
			when(clientsResource.get(REAL_CLIENT_ID)).thenReturn(clientResource);
		}

		@Test
		void shouldReturnClientRole() {
			when(clientResource.roles()).thenReturn(rolesResource);
			when(rolesResource.list()).thenReturn(List.of(roleRepresentation));
			when(roleRepresentation.getName()).thenReturn(UserRepresentationTestFactory.ROLE1);

			Optional<RoleRepresentation> role = service.getClientRole(UserRepresentationTestFactory.ROLE1, REAL_CLIENT_ID, REALM);

			assertThat(role).isPresent().contains(roleRepresentation);
		}

		@Test
		void shouldThrowOnClientNotFound() {
			when(clientsResource.get(REAL_CLIENT_ID)).thenReturn(null);

			assertThrows(KeycloakException.class,
					() -> service.getClientRole(UserRepresentationTestFactory.ROLE1, REAL_CLIENT_ID, REALM));
		}
	}
}
