/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.client;

import static de.ozgcloud.operator.keycloak.client.OzgKeycloakClientSpecTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.keycloak.representations.idm.ClientRepresentation;
import org.keycloak.representations.idm.ProtocolMapperRepresentation;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.operator.keycloak.KeycloakGenericRemoteService;

class KeycloakClientServiceTest {

	private static final String TEST_NAMESPACE = "TestNamespace";
	private static final String REAL_CLIENT_ID = "RealClientId";

	@Spy
	@InjectMocks
	private KeycloakClientService service;

	@Mock
	private KeycloakClientRemoteService remoteService;

	@Mock
	private KeycloakClientMapper mapper;

	@Mock
	private KeycloakGenericRemoteService keycloakGenericRemoteService;
	@Mock
	private ProtocolMapperRepresentationHelper mapperHelper;

	@Nested
	class TestAddClient {

		@Test
		void shouldCallCreateClientMethod() {
			var clientSpec = OzgKeycloakClientSpecTestFactory.create();
			doNothing().when(service).createClient(eq(clientSpec), eq(TEST_NAMESPACE));

			service.createOrUpdateClient(clientSpec, TEST_NAMESPACE);

			verify(service).createClient(eq(clientSpec), eq(TEST_NAMESPACE));
		}

		@Test
		void shouldCallUpdateClientMethod() {
			var clientSpec = OzgKeycloakClientSpecTestFactory.create();
			var clientRepresentation = ClientRepresentationTestFactory.create();
			doNothing().when(service).updateClient(eq(clientRepresentation), eq(clientSpec), eq(TEST_NAMESPACE));
			when(keycloakGenericRemoteService.getByClientId(OzgKeycloakClientSpecTestFactory.CLIENT_ID, TEST_NAMESPACE))
					.thenReturn(Optional.of(clientRepresentation));

			service.createOrUpdateClient(clientSpec, TEST_NAMESPACE);

			verify(service).updateClient(clientRepresentation, clientSpec, TEST_NAMESPACE);
		}
	}

	@DisplayName("Create client")
	@Nested
	class TestCreateClient {

		private final OzgKeycloakClientSpec client = OzgKeycloakClientSpecTestFactory.create();
		private final ClientRepresentation clientRepresentation = ClientRepresentationTestFactory.create();

		@BeforeEach
		void mockMapperHelper() {
			doNothing().when(service).setProtocolMapper(any());

			when(mapper.map(any())).thenReturn(clientRepresentation);
			when(remoteService.createClient(clientRepresentation, TEST_NAMESPACE)).thenReturn(REAL_CLIENT_ID);
		}

		@Test
		void shouldCreateClientIfNotExists() {
			service.createClient(client, TEST_NAMESPACE);

			verify(remoteService).createClient(clientRepresentation, TEST_NAMESPACE);
		}

		@Test
		void shouldCallMapper() {
			service.createClient(client, TEST_NAMESPACE);

			verify(mapper).map(client);
		}

		@Test
		void shouldSetProtocolMapper() {
			service.createClient(client, TEST_NAMESPACE);

			verify(service).setProtocolMapper(clientRepresentation);
		}

		@Test
		void shouldCallAddOrUpdateClientRoles() {
			service.createClient(client, TEST_NAMESPACE);

			verify(service).addOrUpdateClientRoles(client, TEST_NAMESPACE, REAL_CLIENT_ID);
		}
	}

	@DisplayName("Set protocol mapper")
	@Nested
	class TestSetProtocolMapper {

		private final ClientRepresentation clientRepresentation = ClientRepresentationTestFactory.create();
		private final ProtocolMapperRepresentation organisationsEinheitIdMapper = ProtocolMapperRepresentationTestFactory.create();
		private final ProtocolMapperRepresentation ozgCloudUserIdMapper = ProtocolMapperRepresentationTestFactory.create();
		private final ProtocolMapperRepresentation organisationeEinheitIdLdapMapper = ProtocolMapperRepresentationTestFactory.create();
		private final ProtocolMapperRepresentation clientRolesMapper = ProtocolMapperRepresentationTestFactory.create();

		@BeforeEach
		void mockMapperHelper() {
			when(mapperHelper.createOrganisationsEinheitIdMapper()).thenReturn(organisationsEinheitIdMapper);
			when(mapperHelper.createOzgCloudUserIdMapper()).thenReturn(ozgCloudUserIdMapper);
			when(mapperHelper.createOrganisationeEinheitIdLdapMapper()).thenReturn(organisationeEinheitIdLdapMapper);
			when(mapperHelper.createClientRolesMapper()).thenReturn(clientRolesMapper);
		}

		@DisplayName("organisationsEinheitId")
		@Nested
		class TestOrganisationsEinheitIdMapper {

			@Test
			void shouldBeCreated() {
				service.setProtocolMapper(clientRepresentation);

				verify(mapperHelper).createOrganisationsEinheitIdMapper();
			}

			@Test
			void shouldBeSet() {
				service.setProtocolMapper(clientRepresentation);

				assertThat(clientRepresentation.getProtocolMappers()).contains(organisationsEinheitIdMapper);
			}
		}

		@DisplayName("ozgCloudUserId")
		@Nested
		class TestOzgCloudUserIdMapper {

			@Test
			void shouldBeCreated() {
				service.setProtocolMapper(clientRepresentation);

				verify(mapperHelper).createOzgCloudUserIdMapper();
			}

			@Test
			void shouldBeSet() {
				service.setProtocolMapper(clientRepresentation);

				assertThat(clientRepresentation.getProtocolMappers()).contains(ozgCloudUserIdMapper);
			}
		}

		@DisplayName("organisationsEinheitIdLdap")
		@Nested
		class TestOrganisationsEinheitIdLdapMapper {

			@Test
			void shouldBeCreated() {
				service.setProtocolMapper(clientRepresentation);

				verify(mapperHelper).createOrganisationeEinheitIdLdapMapper();
			}

			@Test
			void shouldBeSet() {
				service.setProtocolMapper(clientRepresentation);

				assertThat(clientRepresentation.getProtocolMappers()).contains(organisationeEinheitIdLdapMapper);
			}
		}

		@DisplayName("client roles")
		@Nested
		class TestClientRolesMapper {

			@Test
			void shouldBeCreated() {
				service.setProtocolMapper(clientRepresentation);

				verify(mapperHelper).createClientRolesMapper();
			}

			@Test
			void shouldBeSet() {
				service.setProtocolMapper(clientRepresentation);

				assertThat(clientRepresentation.getProtocolMappers()).contains(clientRolesMapper);
			}
		}
	}

	@Nested
	class TestUpdateClient {

		@Test
		void shouldUpdateClientIfExists() {
			var clientRepresentation = ClientRepresentationTestFactory.create();
			when(mapper.update(any(), any())).thenReturn(clientRepresentation);

			service.updateClient(clientRepresentation, OzgKeycloakClientSpecTestFactory.create(), TEST_NAMESPACE);

			verify(remoteService).updateClient(clientRepresentation, TEST_NAMESPACE);
		}

		@Test
		void shouldCallMapper() {
			var client = OzgKeycloakClientSpecTestFactory.create();
			var clientRepresentation = ClientRepresentationTestFactory.create();

			service.updateClient(clientRepresentation, client, TEST_NAMESPACE);

			verify(mapper).update(clientRepresentation, client);
		}

		@Test
		void shouldCallAddOrUpdateClientRoles() {
			var client = OzgKeycloakClientSpecTestFactory.create();
			var clientRepresentation = mock(ClientRepresentation.class);
			when(clientRepresentation.getId()).thenReturn(REAL_CLIENT_ID);

			service.updateClient(clientRepresentation, client, TEST_NAMESPACE);

			verify(service).addOrUpdateClientRoles(client, TEST_NAMESPACE, REAL_CLIENT_ID);
		}
	}

	@Nested
	class TestAddOrUpdateClientRoles {

		@Test
		void shouldCallRemoteServiceGetRole() {
			service.addOrUpdateClientRoles(OzgKeycloakClientSpecTestFactory.create(), TEST_NAMESPACE, REAL_CLIENT_ID);

			verify(keycloakGenericRemoteService).getClientRole(ROLE_NAME_1, REAL_CLIENT_ID,
					TEST_NAMESPACE);
		}

		@Test
		void shouldCreateRole() {
			when(keycloakGenericRemoteService.getClientRole(ROLE_NAME_1, REAL_CLIENT_ID,
					TEST_NAMESPACE)).thenReturn(Optional.empty());
			var roleRepresentation = RoleRepresentationTestFactory.create();
			when(mapper.mapRole(any())).thenReturn(roleRepresentation);

			service.addOrUpdateClientRoles(OzgKeycloakClientSpecTestFactory.create(), TEST_NAMESPACE, REAL_CLIENT_ID);

			verify(remoteService).addClientRole(roleRepresentation, REAL_CLIENT_ID, TEST_NAMESPACE);
		}

		@Test
		void shouldUpdateRole() {
			var roleRepresentation = RoleRepresentationTestFactory.create();
			when(keycloakGenericRemoteService.getClientRole(ROLE_NAME_1, REAL_CLIENT_ID,
					TEST_NAMESPACE)).thenReturn(Optional.of(roleRepresentation));
			when(mapper.updateRole(any(), any())).thenReturn(roleRepresentation);

			service.addOrUpdateClientRoles(OzgKeycloakClientSpecTestFactory.create(), TEST_NAMESPACE, REAL_CLIENT_ID);

			verify(remoteService).updateClientRole(roleRepresentation, REAL_CLIENT_ID, TEST_NAMESPACE);
		}
	}
}
