/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.keycloak.admin.client.Keycloak;
import org.keycloak.admin.client.resource.RealmsResource;
import org.keycloak.representations.idm.ClientRepresentation;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.operator.keycloak.KeycloakClient;
import de.ozgcloud.operator.keycloak.KeycloakGenericRemoteService;

class KeycloakUserPreconditionServiceTest {

	@Spy
	@InjectMocks
	private KeycloakUserPreconditionService service;

	@Mock
	private KeycloakClient keycloakClient;

	@Mock
	private Keycloak keycloak;

	@Mock
	private RealmsResource realmsResource;

	@Mock
	private KeycloakGenericRemoteService keycloakGenericRemoteService;

	@Nested
	class TestReconcilePreconditionsMet {

		@Test
		void shouldCallRealmExiststest() {
			service.getPreconditionErrors(OzgKeycloakUserTestFactory.create());

			verify(keycloakGenericRemoteService).realmExists(OzgKeycloakUserTestFactory.METADATA_NAMESPACE);
		}

		@Test
		void shouldReturnErrorIfRealmNotExists() {
			doReturn(false).when(keycloakGenericRemoteService).realmExists(anyString());

			var response = service.getPreconditionErrors(OzgKeycloakUserTestFactory.create());

			assertThat(response).isPresent();
		}

		@Test
		void shouldReturnEmptyIfRealmExists() {
			doReturn(true).when(keycloakGenericRemoteService).realmExists(anyString());
			doReturn(Optional.empty()).when(service).clientsExists(any(), any());
			doReturn(Optional.empty()).when(service).groupsExists(any(), any());

			var response = service.getPreconditionErrors(OzgKeycloakUserTestFactory.create());

			assertThat(response).isEmpty();
		}

		@Test
		void shouldCheckIfKeycloakClientExists() {
			doReturn(true).when(keycloakGenericRemoteService).realmExists(anyString());
			doReturn(Optional.empty()).when(service).clientsExists(any(), any());
			var user = OzgKeycloakUserTestFactory.create();

			service.getPreconditionErrors(user);

			verify(service).clientsExists(eq(user), eq(OzgKeycloakUserTestFactory.METADATA_NAMESPACE));
		}

		@Test
		void shouldCheckIfGroupsExists() {
			doReturn(true).when(keycloakGenericRemoteService).realmExists(anyString());
			doReturn(Optional.empty()).when(service).clientsExists(any(), any());
			var user = OzgKeycloakUserTestFactory.create();

			service.getPreconditionErrors(user);

			verify(service).groupsExists(eq(user), eq(OzgKeycloakUserTestFactory.METADATA_NAMESPACE));
		}
	}

	@Nested
	class TestClientExists {

		@Test
		void shouldRespondMissingRealm() {
			when(keycloakGenericRemoteService.getByClientId(any(), any())).thenReturn(Optional.empty());

			var rsp = service.clientsExists(OzgKeycloakUserTestFactory.create(), null);

			assertThat(rsp).isPresent();
		}

		@Test
		void shouldReturnClientExists() {
			when(keycloakGenericRemoteService.getByClientId(any(), any())).thenReturn(Optional.of(mock(ClientRepresentation.class)));

			var rsp = service.clientsExists(OzgKeycloakUserTestFactory.create(), null);

			assertThat(rsp).isEmpty();
		}
	}

	@Nested
	class TestGroupsExists {

		private static final OzgKeycloakUser user = OzgKeycloakUserTestFactory.create();

		@BeforeEach
		void init() {
			doReturn(true).when(keycloakGenericRemoteService).realmExists(anyString());
			doReturn(Optional.empty()).when(service).clientsExists(any(), any());
		}

		@Test
		void shouldReturnMissingGroup() {
			when(keycloakGenericRemoteService.groupExists(KeycloakUserSpecUserTestFactory.GROUP_NAME_1, OzgKeycloakUserTestFactory.METADATA_NAMESPACE)).thenReturn(false);

			var rsp = service.getPreconditionErrors(user);

			assertThat(rsp).isPresent();
		}

		@Test
		void shouldReturnGroupExists() {
			when(keycloakGenericRemoteService.groupExists(KeycloakUserSpecUserTestFactory.GROUP_NAME_1, OzgKeycloakUserTestFactory.METADATA_NAMESPACE)).thenReturn(true);
			when(keycloakGenericRemoteService.groupExists(KeycloakUserSpecUserTestFactory.GROUP_NAME_2, OzgKeycloakUserTestFactory.METADATA_NAMESPACE)).thenReturn(true);

			var rsp = service.getPreconditionErrors(user);

			assertThat(rsp).isEmpty();
		}
	}
}
